@@ -1,11 +1,11 @@
 /* A sort of clone of the cddlib test program `lcdd'.
-   Copyright (C) 2001-2006 Roberto Bagnara <bagnara@cs.unipr.it>
+   Copyright (C) 2001-2008 Roberto Bagnara <bagnara@cs.unipr.it>
 
 This file is part of the Parma Polyhedra Library (PPL).
 
 The PPL is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2 of the License, or (at your
+Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 The PPL is distributed in the hope that it will be useful, but WITHOUT
@@ -44,15 +44,15 @@ namespace PPL = Parma_Polyhedra_Library;
 
 typedef PPL::C_Polyhedron POLYHEDRON_TYPE;
 
-#if !CXX_SUPPORTS_ATTRIBUTE_WEAK
+#if !PPL_CXX_SUPPORTS_ATTRIBUTE_WEAK
 extern "C" void
-set_GMP_memory_allocation_functions(void) {
+ppl_set_GMP_memory_allocation_functions(void) {
 }
 #endif
 
 #elif defined(USE_POLKA)
 
-#include <config.h>
+#include <ppl-config.h>
 #include <gmp.h>
 
 extern "C" {
@@ -73,7 +73,7 @@ typedef poly_t* POLYHEDRON_TYPE;
 
 #elif defined(USE_POLYLIB)
 
-#include <config.h>
+#include <ppl-config.h>
 #include <gmp.h>
 
 extern "C" {
@@ -102,21 +102,21 @@ typedef Polyhedron* POLYHEDRON_TYPE;
 #include <sstream>
 #include <stdexcept>
 
-#ifdef HAVE_GETOPT_H
+#ifdef PPL_HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
-#ifdef HAVE_UNISTD_H
+#ifdef PPL_HAVE_UNISTD_H
 // Include this for `getopt()': especially important if we do not have
 // <getopt.h>.
 # include <unistd.h>
 #endif
 
-#ifdef HAVE_SYS_TIME_H
+#ifdef PPL_HAVE_SYS_TIME_H
 # include <sys/time.h>
 #endif
 
-#ifdef HAVE_SYS_RESOURCE_H
+#ifdef PPL_HAVE_SYS_RESOURCE_H
 // This should be included after <time.h> and <sys/time.h> so as to make
 // sure we have the definitions for, e.g., `ru_utime'.
 # include <sys/resource.h>
@@ -124,16 +124,16 @@ typedef Polyhedron* POLYHEDRON_TYPE;
 
 namespace {
 
-#ifdef HAVE_GETOPT_H
+#ifdef PPL_HAVE_GETOPT_H
 struct option long_options[] = {
   {"max-cpu",        required_argument, 0, 'C'},
   {"max-memory",     required_argument, 0, 'R'},
   {"help",           no_argument,       0, 'h'},
-  {"version",        no_argument,       0, 'V'},
   {"output",         required_argument, 0, 'o'},
   {"timings",        no_argument,       0, 't'},
   {"verbose",        no_argument,       0, 'v'},
 #if defined(USE_PPL)
+  {"version",        no_argument,       0, 'V'},
   {"check",          required_argument, 0, 'c'},
 #endif
   {0, 0, 0, 0}
@@ -141,18 +141,22 @@ struct option long_options[] = {
 #endif
 
 static const char* usage_string
-= "Usage: %s [OPTION]... [FILE]...\n\n"
+= "Usage: %s [OPTION]... [FILE]...\n"
+"Reads an H-representation (resp., a V-representation) of a polyhedron\n"
+"and generates a V-representation (resp., an H-representation) of\n"
+"the same polyhedron.\n\n"
+"Options:\n"
 "  -CSECS, --max-cpu=SECS  limits CPU usage to SECS seconds\n"
 "  -RMB, --max-memory=MB   limits memory usage to MB megabytes\n"
 "  -h, --help              prints this help text to stdout\n"
-"  -V, --version           prints version information to stdout\n"
 "  -oPATH, --output=PATH   appends output to PATH\n"
 "  -t, --timings           prints timings to stderr\n"
 "  -v, --verbose           produces lots of output\n"
 #if defined(USE_PPL)
+"  -V, --version           prints version information to stdout\n"
 "  -cPATH, --check=PATH    checks if the result is equal to what is in PATH\n"
 #endif
-#ifndef HAVE_GETOPT_H
+#ifndef PPL_HAVE_GETOPT_H
 "\n"
 "NOTE: this version does not support long options.\n"
 #endif
@@ -160,9 +164,9 @@ static const char* usage_string
 "Report bugs to <ppl-devel@cs.unipr.it>.\n";
 
 #if defined(USE_PPL)
-#define OPTION_LETTERS "C:R:hVo:tvc:"
+#define OPTION_LETTERS "C:R:ho:tvVc:"
 #else
-#define OPTION_LETTERS "C:R:hVo:tv"
+#define OPTION_LETTERS "C:R:ho:tv"
 #endif
 
 const char* program_name = 0;
@@ -295,7 +299,7 @@ set_alarm_on_cpu_time(const unsigned seconds, sig_handler_type handler) {
   }
 }
 
-#if HAVE_DECL_RLIMIT_AS
+#if PPL_HAVE_DECL_RLIMIT_AS
 void
 limit_virtual_memory(const unsigned bytes) {
   struct rlimit t;
@@ -313,7 +317,7 @@ limit_virtual_memory(const unsigned bytes) {
 void
 limit_virtual_memory(unsigned) {
 }
-#endif // !HAVE_DECL_RLIMIT_AS
+#endif // !PPL_HAVE_DECL_RLIMIT_AS
 
 extern "C" void
 timeout(int) {
@@ -338,7 +342,7 @@ timeout(int) {
 void
 process_options(int argc, char* argv[]) {
   while (true) {
-#ifdef HAVE_GETOPT_H
+#ifdef PPL_HAVE_GETOPT_H
     int option_index = 0;
     int c = getopt_long(argc, argv, OPTION_LETTERS, long_options,
 			&option_index);
@@ -361,11 +365,6 @@ process_options(int argc, char* argv[]) {
       exit(0);
       break;
 
-    case 'V':
-      fprintf(stdout, "%s\n", PPL_VERSION);
-      exit(0);
-      break;
-
     case 'C':
       l = strtol(optarg, &endptr, 10);
       if (*endptr || l < 0)
@@ -394,10 +393,19 @@ process_options(int argc, char* argv[]) {
       verbose = true;
       break;
 
+#if defined(USE_PPL)
+
+    case 'V':
+      fprintf(stdout, "%s\n", PPL_VERSION);
+      exit(0);
+      break;
+
     case 'c':
       check_file_name = optarg;
       break;
 
+#endif
+
     default:
       abort();
     }
@@ -861,7 +869,7 @@ read_polyhedron(std::istream& in, POLYHEDRON_TYPE& ph) {
 
   if (rep == H) {
 #if defined(USE_PPL)
-    ph = PPL::C_Polyhedron(cs);
+    ph = PPL::C_Polyhedron(cs, PPL::Recycle_Input());
 #elif defined(USE_POLKA)
     ph = poly_universe(space_dim);
     ph = poly_add_constraints_lazy(ph, mat);
@@ -877,7 +885,7 @@ read_polyhedron(std::istream& in, POLYHEDRON_TYPE& ph) {
   }
   else {
 #if defined(USE_PPL)
-    ph = PPL::C_Polyhedron(gs);
+    ph = PPL::C_Polyhedron(gs, PPL::Recycle_Input());
 #elif defined(USE_POLKA)
     ph = poly_of_frames(mat);
 #elif defined(USE_POLYLIB)
