@@ -1,11 +1,11 @@
 /* Main program for SWI-Prolog with the PPL statically linked.
-   Copyright (C) 2001-2006 Roberto Bagnara <bagnara@cs.unipr.it>
+   Copyright (C) 2001-2008 Roberto Bagnara <bagnara@cs.unipr.it>
 
 This file is part of the Parma Polyhedra Library (PPL).
 
 The PPL is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2 of the License, or (at your
+Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 The PPL is distributed in the hope that it will be useful, but WITHOUT
@@ -27,6 +27,11 @@ extern "C" install_t uninstall();
 
 int
 main(int argc, char **argv) {
+#if PLVERSION > 50643
+  // Tell SWI-Prolog not to fiddle with GMP allocation functions.
+  PL_action(PL_GMP_SET_ALLOC_FUNCTIONS, FALSE);
+#endif
+
   if (!PL_initialise(argc, argv))
     PL_halt(1);
   PL_install_readline();
