@@ -1,11 +1,11 @@
 /* Grid class implementation: simplify().
-   Copyright (C) 2001-2006 Roberto Bagnara <bagnara@cs.unipr.it>
+   Copyright (C) 2001-2008 Roberto Bagnara <bagnara@cs.unipr.it>
 
 This file is part of the Parma Polyhedra Library (PPL).
 
 The PPL is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2 of the License, or (at your
+Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 The PPL is distributed in the hope that it will be useful, but WITHOUT
@@ -20,200 +20,136 @@ Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1307, USA.
 For the most up-to-date information see the Parma Polyhedra Library
 site: http://www.cs.unipr.it/ppl/ . */
 
-#include <config.h>
+#include <ppl-config.h>
 #include <cassert>
 
 #include "Grid.defs.hh"
 
 namespace Parma_Polyhedra_Library {
 
-#define TRACE(x)
-//#define TRACE(x) x
-
-TRACE(using std::endl);
-TRACE(using std::cerr);
-
-#ifdef STRONG_REDUCTION
-template <typename M, typename R>
 void
-Grid::reduce_reduced(M& sys, dimension_type dim, dimension_type pivot_index,
-		     dimension_type start, dimension_type end,
-		     Dimension_Kinds& dim_kinds, bool generators) {
-  R& pivot = sys[pivot_index];
-
-  TEMP_INTEGER(pivot_dim);
-  pivot_dim = pivot[dim];
-
-  if (pivot_dim == 0)
-    return;
-
-  TEMP_INTEGER(pivot_dim_half);
-  pivot_dim_half = (pivot_dim + 1) / 2;
-  Dimension_Kind row_kind = dim_kinds[dim];
-  Dimension_Kind line_or_equality, virtual_kind;
-  int jump;
-  if (generators) {
-    line_or_equality = LINE;
-    virtual_kind = GEN_VIRTUAL;
-    jump = -1;
-  } else {
-    line_or_equality = EQUALITY;
-    virtual_kind = CON_VIRTUAL;
-    jump = 1;
-  }
-
-  for (dimension_type row_index = pivot_index, kinds_index = dim + jump;
-       row_index-- > 0;
-       kinds_index += jump) {
-    // Move over any virtual rows.
-    while (dim_kinds[kinds_index] == virtual_kind)
-      kinds_index += jump;
-
-    if (row_kind == line_or_equality
-	|| (row_kind == PARAMETER // a.k.a. CONGRUENCE
-	    && dim_kinds[kinds_index] == PARAMETER)) {
-      R& row = sys[row_index];
-
-      TEMP_INTEGER(row_dim);
-      row_dim = row[dim];
-      // num_rows_to_subtract may be positive or negative.
-      TEMP_INTEGER(num_rows_to_subtract);
-      num_rows_to_subtract = row_dim / pivot_dim;
-
-      // Ensure that after subtracting num_rows_to_subtract * r_dim
-      // from row_dim,
-      // -pivot_dim_half < row_dim <= pivot_dim_half.  E.g., if pivot[dim] =
-      // 9, then after strong reduction -5 < row_dim <= 5.
-      Coefficient& row_dim_rem = row_dim;
-      row_dim_rem %= pivot_dim;
-      if (row_dim_rem < 0) {
-	if (row_dim_rem <= -pivot_dim_half)
-	  --num_rows_to_subtract;
-      }
-      else if (row_dim_rem > 0 && row_dim_rem > pivot_dim_half)
-	num_rows_to_subtract++;
-
-      // Subtract num_rows_to_subtract copies of pivot from row i.  Only the
-      // entries from dim need to be subtracted, as the preceding
-      // entries are all zero.
-      // If num_rows_to_subtract is negative, these copies of pivot are
-      // added to row i.
-      if (num_rows_to_subtract != 0)
-	for (dimension_type col = start; col <= end; ++col)
-	  row[col] -= num_rows_to_subtract * pivot[col];
-    }
-  }
-}
-#endif // STRONG_REDUCTION
-
-inline void
 Grid::reduce_line_with_line(Grid_Generator& row, Grid_Generator& pivot,
 			    dimension_type column) {
-  TRACE(cerr << "reduce_line_with_line" << endl);
-
-  TEMP_INTEGER(gcd);
-  gcd_assign(gcd, pivot[column], row[column]);
+  const Coefficient& pivot_column = pivot[column];
+  Coefficient& row_column = row[column];
+  TEMP_INTEGER(reduced_row_col);
+  // Use reduced_row_col temporarily to hold the gcd.
+  gcd_assign(reduced_row_col, pivot_column, row_column);
   // Store the reduced ratio between pivot[column] and row[column].
-  TEMP_INTEGER(red_pivot_col);
-  TEMP_INTEGER(red_row_col);
-  red_pivot_col = pivot[column] / gcd;
-  red_row_col = row[column] / gcd;
+  TEMP_INTEGER(reduced_pivot_col);
+  exact_div_assign(reduced_pivot_col, pivot_column, reduced_row_col);
+  exact_div_assign(reduced_row_col, row_column, reduced_row_col);
   // Multiply row, then subtract from it a multiple of pivot such that
   // the result in row[column] is zero.
-  row[column] = 0;
-  for (dimension_type col = pivot.size() - 2 /* parameter divisor, index */;
+  row_column = 0;
+  // pivot.size() - 1 is the index for the parameter divisor so we
+  // start reducing the line at index pivot.size() - 2.
+  for (dimension_type col = pivot.size() - 2;
        col > column;
-       --col)
-    row[col] = (red_pivot_col * row[col]) - (red_row_col * pivot[col]);
+       --col) {
+    Coefficient& row_col = row[col];
+    row_col *= reduced_pivot_col;
+    sub_mul_assign(row_col, reduced_row_col, pivot[col]);
+  }
 }
 
-inline void
-Grid::reduce_equality_with_equality(Congruence& row, Congruence& pivot,
-				    dimension_type column) {
-  TRACE(cerr << "reduce_equality_with_equality" << endl);
+void
+Grid::reduce_equality_with_equality(Congruence& row,
+				    const Congruence& pivot,
+				    const dimension_type column) {
   // Assume two equalities.
   assert(row.modulus() == 0 && pivot.modulus() == 0);
 
-  TEMP_INTEGER(gcd);
-  gcd_assign(gcd, pivot[column], row[column]);
+  const Coefficient& pivot_column = pivot[column];
+  Coefficient& row_column = row[column];
+  TEMP_INTEGER(reduced_row_col);
+  // Use reduced_row_col temporarily to hold the gcd.
+  gcd_assign(reduced_row_col, pivot_column, row_column);
   // Store the reduced ratio between pivot[column] and row[column].
-  TEMP_INTEGER(red_pivot_col);
-  TEMP_INTEGER(red_row_col);
-  red_pivot_col = pivot[column] / gcd;
-  red_row_col = row[column] / gcd;
+  TEMP_INTEGER(reduced_pivot_col);
+  exact_div_assign(reduced_pivot_col, pivot_column, reduced_row_col);
+  exact_div_assign(reduced_row_col, row_column, reduced_row_col);
   // Multiply row, then subtract from it a multiple of pivot such that
   // the result in row[column] is zero.
-  row[column] = 0;
-  for (dimension_type col = 0; col < column; ++col)
-    row[col] = (red_pivot_col * row[col]) - (red_row_col * pivot[col]);
+  row_column = 0;
+  for (dimension_type col = column; col-- > 0; ) {
+    Coefficient& row_col = row[col];
+    row_col *= reduced_pivot_col;
+    sub_mul_assign(row_col, reduced_row_col, pivot[col]);
+  }
 }
 
 template <typename R>
 void
 Grid::reduce_pc_with_pc(R& row, R& pivot,
-			dimension_type column,
-			dimension_type start,
-			dimension_type end) {
-  TEMP_INTEGER(gcd);
+			const dimension_type column,
+			const dimension_type start,
+			const dimension_type end) {
+  Coefficient& pivot_column = pivot[column];
+  Coefficient& row_column = row[column];
+
   TEMP_INTEGER(s);
   TEMP_INTEGER(t);
-  gcdext_assign(gcd, pivot[column], row[column], s, t);
+  TEMP_INTEGER(reduced_row_col);
+  // Use reduced_row_col temporarily to hold the gcd.
+  gcdext_assign(reduced_row_col, s, t, pivot_column, row_column);
   // Now pivot[column] * s + row[column] * t == gcd.
-  TRACE(cerr << "  gcd " << gcd << ", s " << s << ", t " << t << endl);
 
   // Store the reduced ratio between pivot[column] and row[column].
-  TEMP_INTEGER(red_pivot_col);
-  TEMP_INTEGER(red_row_col);
-  red_pivot_col = pivot[column] / gcd;
-  red_row_col = row[column] / gcd;
-  TRACE(cerr << "  red_pivot_col " << red_pivot_col
-	     << ", red_row_col " << red_row_col << endl);
+  TEMP_INTEGER(reduced_pivot_col);
+  exact_div_assign(reduced_pivot_col, pivot_column, reduced_row_col);
+  pivot_column = reduced_row_col /* gcd */;
+  exact_div_assign(reduced_row_col, row_column, reduced_row_col);
 
   // Multiply row, then subtract from it a multiple of pivot such that
-  // the result in row[column] is zero.  Afterwards, multiply pivot,
+  // the result in row[column] is zero.  Afterward, multiply pivot,
   // then add to it a (possibly negative) multiple of row such that
   // the result in pivot[column] is the smallest possible positive
   // integer.
   assert(pivot.size() > 0);
   assert(row.size() > 0);
-  pivot[column] = gcd;
-  row[column] = 0;
+  row_column = 0;
+  TEMP_INTEGER(old_pivot_col);
   for (dimension_type col = start; col < end; ++col) {
-    TEMP_INTEGER(pivot_col);
-    TEMP_INTEGER(row_col);
-    pivot_col = pivot[col];
-    row_col = row[col];
-    pivot[col] = (s * pivot_col) + (t * row_col);
-    row[col] = (red_pivot_col * row_col) - (red_row_col * pivot_col);
+    Coefficient& pivot_col = pivot[col];
+    old_pivot_col = pivot_col;
+    pivot_col *= s;
+    Coefficient& row_col = row[col];
+    add_mul_assign(pivot_col, t, row_col);
+    row_col *= reduced_pivot_col;
+    sub_mul_assign(row_col, reduced_row_col, old_pivot_col);
   }
 }
 
 void
 Grid::reduce_parameter_with_line(Grid_Generator& row,
-				 Grid_Generator& pivot,
-				 dimension_type column,
+				 const Grid_Generator& pivot,
+				 const dimension_type column,
 				 Grid_Generator_System& sys) {
   // Very similar to reduce_congruence_with_equality below.  Any
   // change here may be needed there too.
-  TRACE(cerr << "reduce_parameter_with_line" << endl);
 
-  dimension_type num_cols = sys.num_columns() - 1 /* parameter divisor */;
+  const Coefficient& pivot_column = pivot[column];
+  Coefficient& row_column = row[column];
+
+  // Subtract one to allow for the parameter divisor column
+  const dimension_type num_columns = sys.num_columns() - 1;
 
   // If the elements at column in row and pivot are the same, then
   // just subtract pivot from row.
-  if (row[column] == pivot[column]) {
-    for (dimension_type col = 0; col < num_cols; ++col)
+  if (row_column == pivot_column) {
+    for (dimension_type col = num_columns; col-- > 0; )
       row[col] -= pivot[col];
     return;
   }
 
-  TEMP_INTEGER(gcd);
-  gcd_assign(gcd, pivot[column], row[column]);
+  TEMP_INTEGER(reduced_row_col);
+  // Use reduced_row_col temporarily to hold the gcd.
+  gcd_assign(reduced_row_col, pivot_column, row_column);
   // Store the reduced ratio between pivot[column] and row[column].
-  TEMP_INTEGER(red_pivot_col);
-  TEMP_INTEGER(red_row_col);
-  red_pivot_col = pivot[column] / gcd;
-  red_row_col = row[column] / gcd;
+  TEMP_INTEGER(reduced_pivot_col);
+  exact_div_assign(reduced_pivot_col, pivot_column, reduced_row_col);
+  exact_div_assign(reduced_row_col, row_column, reduced_row_col);
 
   // Multiply row such that a multiple of pivot can be subtracted from
   // it below to render row[column] zero.  This requires multiplying
@@ -222,72 +158,76 @@ Grid::reduce_parameter_with_line(Grid_Generator& row,
   // Ensure that the multiplier is positive, so that the preceding
   // diagonals (including the divisor) remain positive.  It's safe to
   // swap the signs as row[column] will still come out 0.
-  if (red_pivot_col < 0) {
-    neg_assign(red_pivot_col);
-    neg_assign(red_row_col);
+  if (reduced_pivot_col < 0) {
+    neg_assign(reduced_pivot_col);
+    neg_assign(reduced_row_col);
   }
 #endif
-  for (dimension_type index = 0; index < sys.num_generators(); ++index) {
-    Grid_Generator& row = sys[index];
-    if (row.is_parameter_or_point())
-      for (dimension_type col = 0; col < num_cols; ++col)
-        row[col] *= red_pivot_col;
+  for (dimension_type index = sys.num_rows(); index-- > 0; ) {
+    Grid_Generator& gen = sys[index];
+    if (gen.is_parameter_or_point())
+      for (dimension_type col = num_columns; col-- > 0; )
+        gen[col] *= reduced_pivot_col;
   }
   // Subtract from row a multiple of pivot such that the result in
   // row[column] is zero.
-  row[column] = 0;
-  for (dimension_type col = num_cols - 1; col > column; --col)
-    row[col] -= red_row_col * pivot[col];
+  row_column = 0;
+  for (dimension_type col = num_columns - 1; col > column; --col)
+    sub_mul_assign(row[col], reduced_row_col, pivot[col]);
 }
 
 void
 Grid::reduce_congruence_with_equality(Congruence& row,
-				      Congruence& pivot,
-				      dimension_type column,
+				      const Congruence& pivot,
+				      const dimension_type column,
 				      Congruence_System& sys) {
   // Very similar to reduce_parameter_with_line above.  Any change
   // here may be needed there too.
-  TRACE(cerr << "reduce_congruence_with_equality" << endl);
   assert(row.modulus() > 0 && pivot.modulus() == 0);
 
-  dimension_type num_cols = sys.num_columns();
+  const Coefficient& pivot_column = pivot[column];
+  Coefficient& row_column = row[column];
+
+  dimension_type num_columns = sys.num_columns();
 
   // If the elements at `column' in row and pivot are the same, then
   // just subtract `pivot' from `row'.
-  if (row[column] == pivot[column]) {
-    for (dimension_type col = 0; col < num_cols; ++col)
+  if (row_column == pivot_column) {
+    for (dimension_type col = num_columns; col-- > 0; )
       row[col] -= pivot[col];
     return;
   }
 
-  TEMP_INTEGER(gcd);
-  gcd_assign(gcd, pivot[column], row[column]);
-  TEMP_INTEGER(red_pivot_col);
-  TEMP_INTEGER(red_row_a);
-  red_pivot_col = pivot[column] / gcd;
-  red_row_a = row[column] / gcd;
-  // Ensure that `red_pivot_col' is positive, so that the modulus
+  TEMP_INTEGER(reduced_row_col);
+  // Use reduced_row_col temporarily to hold the gcd.
+  gcd_assign(reduced_row_col, pivot_column, row_column);
+  TEMP_INTEGER(reduced_pivot_col);
+  exact_div_assign(reduced_pivot_col, pivot_column, reduced_row_col);
+  exact_div_assign(reduced_row_col, row_column, reduced_row_col);
+  // Ensure that `reduced_pivot_col' is positive, so that the modulus
   // remains positive when multiplying the proper congruences below.
   // It's safe to swap the signs as row[column] will still come out 0.
-  if (red_pivot_col < 0) {
-    neg_assign(red_pivot_col);
-    neg_assign(red_row_a);
+  if (reduced_pivot_col < 0) {
+    neg_assign(reduced_pivot_col);
+    neg_assign(reduced_row_col);
   }
-  // Multiply `row', including the modulus, by red_pivot_col.  To keep
-  // all the moduli the same this requires multiplying all the other
-  // proper congruences in the same way.
-  for (dimension_type index = 0; index < sys.num_rows(); ++index) {
-    Congruence& row = sys[index];
-    if (row.is_proper_congruence())
-      for (dimension_type col = 0; col < num_cols; ++col)
-        row[col] *= red_pivot_col;
+  // Multiply `row', including the modulus, by reduced_pivot_col.  To
+  // keep all the moduli the same this requires multiplying all the
+  // other proper congruences in the same way.
+  for (dimension_type index = sys.num_rows(); index-- > 0; ) {
+    Congruence& cg = sys[index];
+    if (cg.is_proper_congruence())
+      for (dimension_type col = num_columns; col-- > 0; )
+        cg[col] *= reduced_pivot_col;
   }
-  --num_cols;			// Modulus.
-  row[column] = 0;
+  // Column num_columns contains the modulus, so start at the next
+  // column.
+  --num_columns;
+  row_column = 0;
   // Subtract from row a multiple of pivot such that the result in
   // row[column] is zero.
-  for (dimension_type col = 0; col < column; ++col)
-    row[col] -= red_row_a * pivot[col];
+  for (dimension_type col = column; col-- > 0; )
+    sub_mul_assign(row[col], reduced_row_col, pivot[col]);
 }
 
 #ifndef NDEBUG
@@ -296,7 +236,7 @@ bool
 Grid::rows_are_zero(M& system, dimension_type first,
 		    dimension_type last, dimension_type row_size) {
   while (first <= last) {
-    R& row = system[first++];
+    const R& row = system[first++];
     for (dimension_type col = 0; col < row_size; ++col)
       if (row[col] != 0)
 	return false;
@@ -307,48 +247,37 @@ Grid::rows_are_zero(M& system, dimension_type first,
 
 void
 Grid::simplify(Grid_Generator_System& sys, Dimension_Kinds& dim_kinds) {
-  TRACE(cerr << "==== simplify (reduce) gs:" << endl);
-  TRACE(cerr << "sys:" << endl);
-  TRACE(sys.ascii_dump(cerr));
-  assert(sys.num_generators() > 0);
-  assert(sys.num_columns() > 0); // For reduce_pc_with_pc.
+  assert(!sys.has_no_rows());
+  // For reduce_pc_with_pc.
+  assert(sys.num_columns() > 0);
 
   // Changes here may also be required in the congruence version
   // below.
 
-  dimension_type num_cols = sys.num_columns() - 1 /* parameter divisor */;
+  // Subtract one to allow for the parameter divisor column
+  const dimension_type num_columns = sys.num_columns() - 1;
 
-  if (dim_kinds.size() != num_cols)
-    dim_kinds.resize(num_cols);
+  if (dim_kinds.size() != num_columns)
+    dim_kinds.resize(num_columns);
 
-  dimension_type num_rows = sys.num_generators();
-  TRACE(cerr << "  num_rows " << num_rows << endl);
+  const dimension_type num_rows = sys.num_rows();
 
   // For each dimension `dim' move or construct a row into position
   // `pivot_index' such that the row has zero in all elements
   // following column `dim' and a value other than zero in column
   // `dim'.
   dimension_type pivot_index = 0;
-  for (dimension_type dim = 0; dim < num_cols; ++dim) {
-    TRACE(cerr << "dim " << dim << endl);
-    trace_dim_kinds("  ", dim_kinds);
-
+  for (dimension_type dim = 0; dim < num_columns; ++dim) {
     // Consider the pivot and following rows.
     dimension_type row_index = pivot_index;
-    TRACE(cerr << "  row_index " << row_index << endl);
 
     // Move down over rows which have zero in column `dim'.
-    while (row_index < num_rows && sys[row_index][dim] == 0) {
-      TRACE(cerr << "  .");
+    while (row_index < num_rows && sys[row_index][dim] == 0)
       ++row_index;
-    }
-    TRACE(cerr << endl);
 
-    if (row_index == num_rows) {
+    if (row_index == num_rows)
       // Element in column `dim' is zero in all rows from the pivot.
-      TRACE(cerr << "  Marking virtual row" << endl);
       dim_kinds[dim] = GEN_VIRTUAL;
-    }
     else {
       if (row_index != pivot_index)
 	std::swap(sys[row_index], sys[pivot_index]);
@@ -357,10 +286,8 @@ Grid::simplify(Grid_Generator_System& sys, Dimension_Kinds& dim_kinds) {
 
       // Change the matrix so that the value at `dim' in every row
       // following `pivot_index' is 0, leaving an equivalent grid.
-      TRACE(cerr << "  Reducing all following rows" << endl);
       while (row_index < num_rows - 1) {
 	++row_index;
-	TRACE(cerr << "    row_index " << row_index << endl);
 
 	Grid_Generator& row = sys[row_index];
 
@@ -382,7 +309,7 @@ Grid::simplify(Grid_Generator_System& sys, Dimension_Kinds& dim_kinds) {
 	    reduce_parameter_with_line(row, pivot, dim, sys);
 	  else {
 	    assert(pivot.is_parameter_or_point());
-	    reduce_pc_with_pc(row, pivot, dim, dim + 1, num_cols);
+	    reduce_pc_with_pc(row, pivot, dim, dim + 1, num_columns);
 	  }
 	}
       }
@@ -397,29 +324,28 @@ Grid::simplify(Grid_Generator_System& sys, Dimension_Kinds& dim_kinds) {
 #ifdef STRONG_REDUCTION
       // Ensure a positive follows the leading zeros.
       if (pivot[dim] < 0)
-	pivot.negate(dim, num_cols - 1);
-      TRACE(cerr << "  rr pivot_index " << pivot_index << endl);
-      TRACE(sys.ascii_dump(cerr));
+	pivot.negate(dim, num_columns - 1);
+
       // Factor this row out of the preceding rows.
       reduce_reduced<Grid_Generator_System, Grid_Generator>
-	(sys, dim, pivot_index, dim, num_cols - 1, dim_kinds);
+	(sys, dim, pivot_index, dim, num_columns - 1, dim_kinds);
 #endif
 
       ++pivot_index;
     }
-    TRACE(sys.ascii_dump(cerr));
   }
-  trace_dim_kinds("gs simpl end ", dim_kinds);
 
   // Clip any zero rows from the end of the matrix.
   if (num_rows > pivot_index) {
-    TRACE(cerr << "clipping trailing" << endl);
 #ifndef NDEBUG
-    bool ret = rows_are_zero<Grid_Generator_System,Grid_Generator>
+    const bool ret = rows_are_zero<Grid_Generator_System, Grid_Generator>
       (sys,
-       pivot_index,		 // index of first
-       sys.num_generators() - 1, // index of last
-       sys.num_columns() - 1);	 // row size
+       // index of first
+       pivot_index,
+       // index of last
+       sys.num_rows() - 1,
+       // row size
+       sys.num_columns() - 1);
     assert(ret == true);
 #endif
     sys.erase_to_end(pivot_index);
@@ -429,30 +355,24 @@ Grid::simplify(Grid_Generator_System& sys, Dimension_Kinds& dim_kinds) {
 
   // Ensure that the parameter divisors are the same as the system
   // divisor.
-  TRACE(cerr << "updating param divisors" << endl);
-  Coefficient_traits::const_reference system_divisor = sys[0][0];
-  for (dimension_type row = 1, dim = 1, num_cols = sys.num_columns() - 1;
-       dim < num_cols;
-       ++dim)
+  const Coefficient& system_divisor = sys[0][0];
+  for (dimension_type row = sys.num_rows() - 1,
+	 dim = sys.num_columns() - 2;
+       dim > 0; --dim)
     switch (dim_kinds[dim]) {
     case PARAMETER:
-      sys[row].divisor() = system_divisor;
+      sys[row].set_divisor(system_divisor);
     case LINE:
-      ++row;
+      --row;
     case GEN_VIRTUAL:
       break;
     }
 
   assert(sys.OK());
-
-  TRACE(cerr << "---- simplify (reduce) gs done." << endl);
 }
 
 bool
 Grid::simplify(Congruence_System& sys, Dimension_Kinds& dim_kinds) {
-  TRACE(cerr << "======== simplify (reduce) cgs:" << endl);
-  TRACE(cerr << "sys:" << endl);
-  TRACE(sys.ascii_dump(cerr));
   assert(sys.num_columns() > 2);
 
   // Changes here may also be required in the generator version above.
@@ -461,40 +381,30 @@ Grid::simplify(Congruence_System& sys, Dimension_Kinds& dim_kinds) {
   //       added to con_sys.
   sys.normalize_moduli();
 
-  dimension_type num_cols = sys.num_columns() - 1 /* modulus */;
+  const dimension_type num_columns = sys.num_columns() - 1 /* modulus */;
 
-  if (dim_kinds.size() != num_cols)
-    dim_kinds.resize(num_cols);
+  if (dim_kinds.size() != num_columns)
+    dim_kinds.resize(num_columns);
 
-  dimension_type num_rows = sys.num_rows();
-  TRACE(cerr << "  num_rows " << num_rows << endl);
+  const dimension_type num_rows = sys.num_rows();
 
   // For each dimension `dim' move or construct a row into position
   // `pivot_index' such that the row has a value of zero in all
   // elements preceding column `dim' and some other value in column
   // `dim'.
   dimension_type pivot_index = 0;
-  for (dimension_type dim = num_cols; dim-- > 0; ) {
-    TRACE(cerr << "dim " << dim << endl);
-    trace_dim_kinds("  ", dim_kinds);
-
+  for (dimension_type dim = num_columns; dim-- > 0; ) {
     // Consider the pivot and following rows.
     dimension_type row_index = pivot_index;
-    TRACE(cerr << "  row_index " << row_index << endl);
 
     // Move down over rows which have zero in column `dim'.
-    while (row_index < num_rows && sys[row_index][dim] == 0) {
-      TRACE(cerr << "  .");
+    while (row_index < num_rows && sys[row_index][dim] == 0)
       ++row_index;
-    }
-    TRACE(cerr << endl);
 
-    if (row_index == num_rows) {
+    if (row_index == num_rows)
       // Element in column `dim' is zero in all rows from the pivot,
       // or `sys' is empty of rows.
-      TRACE(cerr << "  Marking virtual row" << endl);
       dim_kinds[dim] = CON_VIRTUAL;
-    }
     else {
       // row_index != num_rows
       if (row_index != pivot_index)
@@ -504,10 +414,8 @@ Grid::simplify(Congruence_System& sys, Dimension_Kinds& dim_kinds) {
 
       // Change the matrix so that the value at `dim' in every row
       // following `pivot_index' is 0, leaving an equivalent grid.
-      TRACE(cerr << "  Reducing all following rows" << endl);
       while (row_index < num_rows - 1) {
 	++row_index;
-	TRACE(cerr << "    row_index " << row_index << endl);
 
 	Congruence& row = sys[row_index];
 
@@ -551,32 +459,10 @@ Grid::simplify(Congruence_System& sys, Dimension_Kinds& dim_kinds) {
 #endif
       ++pivot_index;
     }
-    TRACE(sys.ascii_dump(cerr));
-  } // end for (dimension_type dim = num_cols; dim-- > 0; )
+  } // end for (dimension_type dim = num_columns; dim-- > 0; )
 
-  dimension_type& reduced_num_rows = pivot_index; // For clearer naming.
-
-  // Clip any zero rows from the end of the matrix.
-  if (num_rows > 1 && num_rows > reduced_num_rows) {
-    TRACE(cerr << "clipping trailing" << endl);
-#ifndef NDEBUG
-    bool ret = rows_are_zero<Congruence_System,Congruence>
-      (sys,
-       reduced_num_rows,	// index of first
-       num_rows - 1,		// index of last
-       num_cols);		// row size
-    assert(ret == true);
-#endif
-    // Don't erase the last row as this will be changed to the integrality row.
-    // FIXME Simplify and improve code if possible.
-    if (reduced_num_rows > 0)
-      sys.erase_to_end(reduced_num_rows);
-    else
-      sys.erase_to_end(1);
-  }
-
-  assert(sys.num_rows() == reduced_num_rows
-	 || (sys.num_rows() == 1 && reduced_num_rows == 0));
+  // For clearer naming.
+  dimension_type& reduced_num_rows = pivot_index;
 
   if (reduced_num_rows > 0) {
     // If the last row is false then make it the equality 1 = 0, and
@@ -600,28 +486,59 @@ Grid::simplify(Congruence_System& sys, Dimension_Kinds& dim_kinds) {
       std::swap(sys.rows[0], sys.rows.back());
       sys.erase_to_end(1);
 
-      trace_dim_kinds("cgs simpl end ", dim_kinds);
       assert(sys.OK());
-      TRACE(cerr << "---- simplify (reduce) cgs done (empty)." << endl);
       return true;
     }
   }
-  else if (num_rows > 0) {
-    assert(sys.num_rows() == 1);
-    // All columns up to the modulus column must have been zero, set
-    // up the integrality congruence.
+  else {
+    // Either sys is empty (it defines the universe) or every column
+    // before the modulus column contains only zeroes.
+
+    // Set up the integrality congruence.
     dim_kinds[0] = PROPER_CONGRUENCE;
-    sys[0][num_cols] = 1;
+    if (num_rows == 0) {
+      sys.add_zero_rows(1,
+			Linear_Row::Flags(NECESSARILY_CLOSED,
+					  Linear_Row::RAY_OR_POINT_OR_INEQUALITY));
+      Congruence& cg = sys[0];
+      cg[num_columns] = 1;
+      cg[0] = 1;
+
+      assert(sys.OK());
+      return false;
+    }
+    sys[0][num_columns] = 1;
+    // Ensure that, after any zero row clipping below, a single row
+    // will remain for the integrality congruence.
     reduced_num_rows = 1;
   }
 
+  // Clip any zero rows from the end of the matrix.
+  if (num_rows > 1 && num_rows > reduced_num_rows) {
+#ifndef NDEBUG
+    const bool ret = rows_are_zero<Congruence_System, Congruence>
+      (sys,
+       // index of first
+       reduced_num_rows,
+       // index of last
+       num_rows - 1,
+       // row size
+       num_columns);
+    assert(ret == true);
+#endif
+    sys.erase_to_end(reduced_num_rows);
+  }
+
+  assert(sys.num_rows() == reduced_num_rows);
+
   // Ensure that the last row is the integrality congruence.
-  dimension_type mod_index = num_cols;
+  const dimension_type mod_index = num_columns;
   if (dim_kinds[0] == CON_VIRTUAL) {
     // The last row is virtual, append the integrality congruence.
     dim_kinds[0] = PROPER_CONGRUENCE;
-    sys.add_zero_rows(1, Linear_Row::Flags(NECESSARILY_CLOSED,
-					   Linear_Row::RAY_OR_POINT_OR_INEQUALITY));
+    sys.add_zero_rows(1,
+		      Linear_Row::Flags(NECESSARILY_CLOSED,
+					Linear_Row::RAY_OR_POINT_OR_INEQUALITY));
     Congruence& new_last_row = sys[reduced_num_rows];
     new_last_row[mod_index] = 1;
     // Try use an existing modulus.
@@ -649,12 +566,8 @@ Grid::simplify(Congruence_System& sys, Dimension_Kinds& dim_kinds) {
     (sys, 0, reduced_num_rows - 1, 0, 0, dim_kinds, false);
 #endif
 
-  trace_dim_kinds("cgs simpl end ", dim_kinds);
   assert(sys.OK());
-  TRACE(cerr << "---- simplify (reduce) cgs done." << endl);
   return false;
 }
 
-#undef TRACE
-
 } // namespace Parma_Polyhedra_Library
