@@ -1,11 +1,11 @@
 /* Linear_System class implementation (non-inline functions).
-   Copyright (C) 2001-2006 Roberto Bagnara <bagnara@cs.unipr.it>
+   Copyright (C) 2001-2008 Roberto Bagnara <bagnara@cs.unipr.it>
 
 This file is part of the Parma Polyhedra Library (PPL).
 
 The PPL is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2 of the License, or (at your
+Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 The PPL is distributed in the hope that it will be useful, but WITHOUT
@@ -20,12 +20,12 @@ Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1307, USA.
 For the most up-to-date information see the Parma Polyhedra Library
 site: http://www.cs.unipr.it/ppl/ . */
 
-#include <config.h>
+#include <ppl-config.h>
 
 #include "Linear_System.defs.hh"
 #include "Coefficient.defs.hh"
 #include "Row.defs.hh"
-#include "Saturation_Matrix.defs.hh"
+#include "Bit_Matrix.defs.hh"
 #include "Scalar_Products.defs.hh"
 #include <algorithm>
 #include <iostream>
@@ -131,7 +131,7 @@ PPL::Linear_System::ascii_dump(std::ostream& s) const {
     x[i].ascii_dump(s);
 }
 
-PPL_OUTPUT_DEFINITIONS_ASCII_ONLY(Linear_System);
+PPL_OUTPUT_DEFINITIONS_ASCII_ONLY(Linear_System)
 
 bool
 PPL::Linear_System::ascii_load(std::istream& s) {
@@ -173,7 +173,7 @@ PPL::Linear_System::ascii_load(std::istream& s) {
     if (!x[row].ascii_load(s))
       return false;
 
-  // Check for well-formedness.
+  // Check invariants.
   assert(OK(true));
   return true;
 }
@@ -200,16 +200,14 @@ PPL::Linear_System::insert(const Linear_Row& r) {
       swap_columns(old_num_columns - 1, r_size - 1);
     add_row(r);
   }
-  else if (r_size < old_num_columns)
-    if (is_necessarily_closed() || old_num_rows == 0)
-      add_row(Linear_Row(r, old_num_columns, row_capacity));
-    else {
-      // Create a resized copy of the row (and move the epsilon
-      // coefficient to its last position).
-      Linear_Row tmp_row(r, old_num_columns, row_capacity);
+  else if (r_size < old_num_columns) {
+    // Create a resized copy of the row.
+    Linear_Row tmp_row(r, old_num_columns, row_capacity);
+    // If needed, move the epsilon coefficient to the last position.
+    if (!is_necessarily_closed())
       std::swap(tmp_row[r_size - 1], tmp_row[old_num_columns - 1]);
-      add_row(tmp_row);
-    }
+    add_row(tmp_row);
+  }
   else
     // Here r_size == old_num_columns.
     add_row(r);
@@ -289,11 +287,11 @@ PPL::Linear_System::add_rows(const Linear_System& y) {
   assert(num_pending_rows() == 0);
 
   // Adding no rows is a no-op.
-  if (y.num_rows() == 0)
+  if (y.has_no_rows())
     return;
 
   // Check if sortedness is preserved.
-  if (is_sorted())
+  if (is_sorted()) {
     if (!y.is_sorted() || y.num_pending_rows() > 0)
       set_sorted(false);
     else {
@@ -302,6 +300,7 @@ PPL::Linear_System::add_rows(const Linear_System& y) {
       if (n_rows > 0)
 	set_sorted(compare((*this)[n_rows-1], y[0]) <= 0);
     }
+  }
 
   // Add the rows of `y' as if they were pending.
   add_pending_rows(y);
@@ -340,7 +339,7 @@ PPL::Linear_System::sort_rows(const dimension_type first_row,
   std::vector<Row>::iterator new_last = swapping_unique(first, last);
   // Finally, remove duplicates.
   rows.erase(new_last, last);
-  // NOTE: we cannot check for well-formedness of the system here,
+  // NOTE: we cannot check all invariants of the system here,
   // because the caller still has to update `index_first_pending'.
 }
 
@@ -453,28 +452,31 @@ PPL::Linear_System::add_pending_row(const Linear_Row::Flags flags) {
 void
 PPL::Linear_System::normalize() {
   Linear_System& x = *this;
+  const dimension_type nrows = x.num_rows();
   // We normalize also the pending rows.
-  for (dimension_type i = num_rows(); i-- > 0; )
+  for (dimension_type i = nrows; i-- > 0; )
     x[i].normalize();
-  set_sorted(false);
+  set_sorted(nrows <= 1);
 }
 
 void
 PPL::Linear_System::strong_normalize() {
   Linear_System& x = *this;
+  const dimension_type nrows = x.num_rows();
   // We strongly normalize also the pending rows.
-  for (dimension_type i = num_rows(); i-- > 0; )
+  for (dimension_type i = nrows; i-- > 0; )
     x[i].strong_normalize();
-  set_sorted(false);
+  set_sorted(nrows <= 1);
 }
 
 void
 PPL::Linear_System::sign_normalize() {
   Linear_System& x = *this;
+  const dimension_type nrows = x.num_rows();
   // We sign-normalize also the pending rows.
   for (dimension_type i = num_rows(); i-- > 0; )
     x[i].sign_normalize();
-  set_sorted(false);
+  set_sorted(nrows <= 1);
 }
 
 /*! \relates Parma_Polyhedra_Library::Linear_System */
@@ -498,7 +500,7 @@ PPL::operator==(const Linear_System& x, const Linear_System& y) {
 }
 
 void
-PPL::Linear_System::sort_and_remove_with_sat(Saturation_Matrix& sat) {
+PPL::Linear_System::sort_and_remove_with_sat(Bit_Matrix& sat) {
   Linear_System& sys = *this;
   // We can only sort the non-pending part of the system.
   assert(sys.first_pending_row() == sat.num_rows());
@@ -508,11 +510,11 @@ PPL::Linear_System::sort_and_remove_with_sat(Saturation_Matrix& sat) {
   }
 
   // First, sort `sys' (keeping `sat' consistent) without removing duplicates.
-  With_Saturation_Matrix_iterator first(sys.rows.begin(), sat.rows.begin());
-  With_Saturation_Matrix_iterator last = first + sat.num_rows();
+  With_Bit_Matrix_iterator first(sys.rows.begin(), sat.rows.begin());
+  With_Bit_Matrix_iterator last = first + sat.num_rows();
   swapping_sort(first, last, Row_Less_Than());
   // Second, move duplicates in `sys' to the end (keeping `sat' consistent).
-  With_Saturation_Matrix_iterator new_last = swapping_unique(first, last);
+  With_Bit_Matrix_iterator new_last = swapping_unique(first, last);
 
   const dimension_type num_duplicates = last - new_last;
   const dimension_type new_first_pending_row
@@ -594,6 +596,7 @@ PPL::Linear_System
   // lines or equalities, all of which occur before the first ray
   // or point or inequality.
   assert(x.OK(true));
+  assert(x.num_columns() >= 1);
   assert(x.num_pending_rows() == 0);
   assert(n_lines_or_equalities <= x.num_lines_or_equalities());
 #ifndef NDEBUG
@@ -605,7 +608,7 @@ PPL::Linear_System
   const dimension_type ncols = x.num_columns();
   // Trying to keep sortedness.
   bool still_sorted = x.is_sorted();
-  // This deque of booleans will be used to flag those rows that,
+  // This deque of Booleans will be used to flag those rows that,
   // before exiting, need to be re-checked for sortedness.
   std::deque<bool> check_for_sortedness;
   if (still_sorted)
@@ -672,7 +675,7 @@ PPL::Linear_System
   }
 
   // Trying to keep sortedness.
-  for (dimension_type i = 0; still_sorted && i < nrows-1; ++i)
+  for (dimension_type i = 0; still_sorted && i+1 < nrows; ++i)
     if (check_for_sortedness[i])
       // Have to check sortedness of `x[i]' with respect to `x[i+1]'.
       still_sorted = (compare(x[i], x[i+1]) <= 0);
@@ -703,7 +706,7 @@ PPL::Linear_System::simplify() {
       }
       ++n_lines_or_equalities;
     }
-  // Apply Gaussian's elimination to the subsystem of lines/equalities.
+  // Apply Gaussian elimination to the subsystem of lines/equalities.
   const dimension_type rank = x.gauss(n_lines_or_equalities);
   // Eliminate any redundant line/equality that has been detected.
   if (rank < n_lines_or_equalities) {
@@ -844,7 +847,7 @@ PPL::Linear_System::OK(const bool check_strong_normalized) const {
 
   // An empty system is OK,
   // unless it is an NNC system with exactly one column.
-  if (num_rows() == 0)
+  if (has_no_rows()) {
     if (is_necessarily_closed() || num_columns() != 1)
       return true;
     else {
@@ -853,6 +856,7 @@ PPL::Linear_System::OK(const bool check_strong_normalized) const {
 #endif
       return false;
     }
+  }
 
   // A non-empty system will contain constraints or generators; in
   // both cases it must have at least one column for the inhomogeneous
