@@ -1,11 +1,11 @@
 /* Polyhedron class implementation: minimize() and add_and_minimize().
-   Copyright (C) 2001-2006 Roberto Bagnara <bagnara@cs.unipr.it>
+   Copyright (C) 2001-2008 Roberto Bagnara <bagnara@cs.unipr.it>
 
 This file is part of the Parma Polyhedra Library (PPL).
 
 The PPL is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2 of the License, or (at your
+Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 The PPL is distributed in the hope that it will be useful, but WITHOUT
@@ -20,10 +20,10 @@ Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1307, USA.
 For the most up-to-date information see the Parma Polyhedra Library
 site: http://www.cs.unipr.it/ppl/ . */
 
-#include <config.h>
+#include <ppl-config.h>
 #include "Linear_Row.defs.hh"
 #include "Linear_System.defs.hh"
-#include "Saturation_Matrix.defs.hh"
+#include "Bit_Matrix.defs.hh"
 #include "Polyhedron.defs.hh"
 #include <stdexcept>
 
@@ -69,7 +69,7 @@ bool
 PPL::Polyhedron::minimize(const bool con_to_gen,
 			  Linear_System& source,
 			  Linear_System& dest,
-			  Saturation_Matrix& sat) {
+			  Bit_Matrix& sat) {
   // Topologies have to agree.
   assert(source.topology() == dest.topology());
   // `source' cannot be empty: even if it is an empty constraint system,
@@ -77,7 +77,7 @@ PPL::Polyhedron::minimize(const bool con_to_gen,
   // the positive constraint. It also cannot be an empty generator system,
   // since this function is always called starting from a non-empty
   // polyhedron.
-  assert(source.num_rows() > 0);
+  assert(!source.has_no_rows());
 
   // Sort the source system, if necessary.
   if (!source.is_sorted())
@@ -125,7 +125,7 @@ PPL::Polyhedron::minimize(const bool con_to_gen,
   //       portion of `tmp_sat' is the sub-matrix consisting of
   //       the first 0 columns: thus the relevant portion correctly
   //       characterizes the initial saturation information.
-  Saturation_Matrix tmp_sat(dest_num_rows, source.num_rows());
+  Bit_Matrix tmp_sat(dest_num_rows, source.num_rows());
 
   // By invoking the function conversion(), we populate `dest' with
   // the generators characterizing the polyhedron described by all
@@ -218,7 +218,7 @@ PPL::Polyhedron::minimize(const bool con_to_gen,
   pair (\p source1, \p dest) and a system of new constraints \p source2,
   modifies \p source1 by adding to it the constraints of \p source2 that
   are not in \p source1. Then, by invoking
-  <CODE>add_and_minimize(bool, Linear_System&, Linear_System&, Saturation_Matrix&)</CODE>,
+  <CODE>add_and_minimize(bool, Linear_System&, Linear_System&, Bit_Matrix&)</CODE>,
   processes the added constraints obtaining a new DD pair.
 
   This method treats also the dual case, i.e., adding new generators to
@@ -233,10 +233,10 @@ bool
 PPL::Polyhedron::add_and_minimize(const bool con_to_gen,
 				  Linear_System& source1,
 				  Linear_System& dest,
-				  Saturation_Matrix& sat,
+				  Bit_Matrix& sat,
 				  const Linear_System& source2) {
   // `source1' and `source2' cannot be empty.
-  assert(source1.num_rows() > 0 && source2.num_rows() > 0);
+  assert(!source1.has_no_rows() && !source2.has_no_rows());
   // `source1' and `source2' must have the same number of columns
   // to be merged.
   assert(source1.num_columns() == source2.num_columns());
@@ -334,7 +334,7 @@ bool
 PPL::Polyhedron::add_and_minimize(const bool con_to_gen,
 				  Linear_System& source,
 				  Linear_System& dest,
-				  Saturation_Matrix& sat) {
+				  Bit_Matrix& sat) {
   assert(source.num_pending_rows() > 0);
   assert(source.num_columns() == dest.num_columns());
   assert(source.is_sorted());
